<?php
/**
 * DyaPress e-Commerce Admin.
 *
 * @class    DyaPressECommerceAdmin
 * @package  DyaPressECommerce/Admin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * WordPress DyaPress e-Commerce plugin :
 * Copyright (C) 2018-2020      François Pons        <fpons@aytechnet.fr>
 *
 * Please see https://dyapress.com/
 */

defined( 'ABSPATH' ) or die( 'Oops' );

class DyaPressECommerceAdmin {
    public static function add_pages() {
	$admin_orders_page = array( __CLASS__, 'admin_orders_page' );

	global $hook_orders_page;

	$hook_orders_page = dyapress_ecommerce()->add_menu_page( __( 'Dyapress Premium Orders', 'dyapress-ecommerce' ), __( 'DyaPress Premium', 'dyapress-ecommerce' ), 'manage_options', $admin_orders_page );

	dyapress_ecommerce()->add_submenu_page( __( 'DyaPress Premium orders', 'dyapress-ecommerce' ), __( 'Orders', 'dyapress-ecommerce' ), 'manage_options', '', $admin_orders_page, 10 );
	add_action( 'load-'.$hook_orders_page, array( __CLASS__, 'admin_orders_page_option' ) );
	add_filter( 'screen_settings', array( __CLASS__, 'display_screen_options' ), 10, 2 );

	/* Will be added soon :
	dyapress_ecommerce()->add_submenu_page( __( 'DyaPress Premium import', 'dyapress-ecommerce' ), __( 'Import', 'dyapress-ecommerce' ), 'manage_options', 'dyapress_ecommerce_import', array( __CLASS__, 'admin_import_page' ), 70 );
	dyapress_ecommerce()->add_submenu_page( __( 'DyaPress Premium logs', 'dyapress-ecommerce' ), __( 'Logs', 'dyapress-ecommerce' ), 'manage_options', 'dyapress_ecommerce_logs', array( __CLASS__, 'admin_logs_page' ), 80 );
	 */
	dyapress_ecommerce()->add_submenu_page( __( 'DyaPress Premium settings', 'dyapress-ecommerce' ), __( 'Settings', 'dyapress-ecommerce' ), 'manage_options', 'settings', array( __CLASS__, 'admin_settings_page' ), 90 );

	dyapress_ecommerce()->add_settings_tab( __( 'API', 'dyapress-ecommerce' ), 'api', array( __CLASS__, 'admin_settings_tab_api' ), 0 );
	dyapress_ecommerce()->add_settings_tab( __( 'Stripe', 'dyapress-ecommerce' ), 'stripe', array( __CLASS__, 'admin_settings_tab_stripe' ), 10 );
	dyapress_ecommerce()->add_settings_tab( __( 'Account', 'dyapress-ecommerce' ), 'account', array( __CLASS__, 'admin_settings_tab_account' ), 20 );

	do_action( 'dyapress_ecommerce_admin_add_pages' );
    }

    private static function manage_orders( $orders ) {
	$result = dyapress_premium()->manage_orders( $orders );
	$message = '';

	if ( !empty( $result ) ) {
	    if ( is_array( $result->unconfirmed ) && count( $result->unconfirmed ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unconfirmed ) > 1 ? __( 'The orders %s are not confirmed.', 'dyapress-ecommerce' ) : __( 'The order %s is not confirmed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unconfirmed ) ).'</p>';
	    }
	    if ( is_array( $result->unpayable ) && count( $result->unpayable ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unpayable ) > 1 ? __( 'The orders %s are left unpaid.', 'dyapress-ecommerce' ) : __( 'The order %s is left unpaid', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unpayable ) ).'</p>';
	    }
	    if ( is_array( $result->management ) && count( $result->management ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->management ) > 1 ? __( 'The orders %s are being managed.', 'dyapress-ecommerce' ) : __( 'The order %s is being managed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->management ) ).'</p>';
	    }
	}

	return $message;
    }

    private static function prepare_orders( $orders ) {
	$result = dyapress_premium()->prepare_orders( $orders );
	$message = '';

	if ( !empty( $result ) ) {
	    if ( is_array( $result->unconfirmed ) && count( $result->unconfirmed ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unconfirmed ) > 1 ? __( 'The orders %s are not confirmed.', 'dyapress-ecommerce' ) : __( 'The order %s is not confirmed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unconfirmed ) ).'</p>';
	    }
	    if ( is_array( $result->unpayable ) && count( $result->unpayable ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unpayable ) > 1 ? __( 'The orders %s are left unpaid.', 'dyapress-ecommerce' ) : __( 'The order %s is left unpaid', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unpayable ) ).'</p>';
	    }
	    if ( is_array( $result->closed ) && count( $result->closed ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->closed ) > 1 ? __( 'The orders %s are already closed.', 'dyapress-ecommerce' ) : __( 'The order %s is already closed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->closed ) ).'</p>';
	    }
	    if ( is_array( $result->preparation ) && count( $result->preparation ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->preparation ) > 1 ? __( 'The orders %s are being prepared.', 'dyapress-ecommerce' ) : __( 'The order %s is being prepared', 'dyapress-ecommerce' ),
		    implode( ', ', $result->preparation ) ).'</p>';
	    }
	}

	return $message;
    }

    private static function complete_orders( $orders ) {
	$result = dyapress_premium()->complete_orders( $orders );
	$message = '';

	if ( !empty( $result ) ) {
	    if ( is_array( $result->unconfirmed ) && count( $result->unconfirmed ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unconfirmed ) > 1 ? __( 'The orders %s are not confirmed.', 'dyapress-ecommerce' ) : __( 'The order %s is not confirmed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unconfirmed ) ).'</p>';
	    }
	    if ( is_array( $result->unpaid ) && count( $result->unpaid ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->unpaid ) > 1 ? __( 'The orders %s are unpaid.', 'dyapress-ecommerce' ) : __( 'The order %s is unpaid', 'dyapress-ecommerce' ),
		    implode( ', ', $result->unpaid ) ).'</p>';
	    }
	    if ( is_array( $result->completed ) && count( $result->completed ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->completed ) > 1 ? __( 'The orders %s are already completed.', 'dyapress-ecommerce' ) : __( 'The order %s is already completed', 'dyapress-ecommerce' ),
		    implode( ', ', $result->completed ) ).'</p>';
	    }
	    if ( is_array( $result->completion ) && count( $result->completion ) > 0 ) {
		$message .= '<p>'.sprintf( count( $result->completion ) > 1 ? __( 'The orders %s are being finalized.', 'dyapress-ecommerce' ) : __( 'The order %s is being finalized', 'dyapress-ecommerce' ),
		    implode( ', ', $result->completion ) ).'</p>';
	    }
	}

	return $message;
    }

    public static function admin_orders_page_option() {
	$option_name = 'dyapress_ecommerce_orders_per_page';
	$args = array(
	    'default' => get_option( 'posts_per_page'),
	    'option' => $option_name,
	);

	add_screen_option( $option_name, $args );
    }

    public static function display_screen_options( $status, $args ) {
	global $hook_orders_page;

	$screen = get_current_screen();

	if ( is_object( $screen ) && $screen->id == $hook_orders_page ) {
	    $user = get_current_user_id();
	    $option = $screen->get_option( 'dyapress_ecommerce_orders_per_page', 'option' );
	    $limit = (int)get_user_meta( $user, $option, true );

	    if ( empty( $limit ) || $limit < 1 )
		$limit = (int)$screen->get_option( 'dyapress_ecommerce_orders_per_page', 'default' );

	    return $status . '<fieldset class="screen-options"><legend>'.__( 'Paginate', 'dyapress-ecommerce' ).'</legend><label for="dyapress_ecommerce_orders_per_page">'.__( 'Number of orders displayed per page:', 'dyapress-ecommerce' ).'</label>&nbsp;<input type="number" step="1" min="1" max="9999" class="orders-per-page" name="wp_screen_options[value]" id="dyapress_ecommerce_orders_per_page" maxlength="4" value="'.esc_html( $limit ).'"><input type="hidden" name="wp_screen_options[option]" value="dyapress_ecommerce_orders_per_page"></fieldset><p class="submit"><input type="submit" name="screen-options-apply" id="screen-options-apply" class="button button-primary" value="'.__( 'Apply', 'dyapress-ecommerce' ).'"></p>';
	}

	return;
    }

    private static function admin_orders_tablenav( $pos, $args, $result ) {
?>
		<div class="tablenav top">
		    <div class="alignleft actions blukactions">
			<label for="bulk-action-selector-<?php echo $pos; ?>" class="screen-reader-text"><?php _e( 'Select group action', 'dyapress-ecommerce' ); ?></label>
			<select name="action<?php if ( $pos != 'top' ) echo '2'; ?>" id="bulk-action-selector-<?php echo $pos; ?>">
			    <option value=""><?php _e( 'Group actions', 'dyapress-ecommerce' ); ?></option>
			    <option value="manage"><?php _e( 'Manage orders', 'dyapress-ecommerce' ); ?></option>
			    <option value="prepare"><?php _e( 'Capture payments and prepare orders', 'dyapress-ecommerce' ); ?></option>
			    <option value="complete"><?php _e( 'Complete orders and create invoices', 'dyapress-ecommerce' ); ?></option>
			</select>
			<input type="submit" id="doaction<?php if ( $pos != 'top' ) echo '2'; ?>" class="button action" value="<?php _e( 'Apply', 'dyapress-ecommerce' ); ?>">
		    </div>
		    <div class="tablenav-pages">
			<span class="displaying-num"><?php if ( $result->total > 1 ) printf( __( '%d orders', 'dyapress-ecommerce' ), $result->total ); else printf( __( '%d order', 'dyapress-ecommerce' ), $result->total ); ?></span><?php
	if ( $result->max_num_pages > 1 ) {
	    if ( !empty( $args['paged'] ) )
		$paged = $args['paged'];
	    else
		$paged = 1;

	    $url = dyapress_ecommerce()->admin_url();
	    if ( !empty( $args['order_state'] ) ) {
		$order_state = $args['order_state'];
		if ( $order_state != 'all' )
		    $url .= '&order_state='.$order_state;
	    }
?>
			<span class="pagination-links"><?php
	    if ( $paged > 1 ) { ?>
			    <a class="next-page button" href="<?php echo $url.'&paged=1'; ?>">
				<span class="screen-reader-text"><?php _e( 'First page', 'dyapress-ecommerce' ); ?></span>
				<span aria-hidden="true">«</span>
			    </a>
			    <a class="last-page button" href="<?php echo $url.'&paged='.($paged-1); ?>">
				<span class="screen-reader-text"><?php _e( 'Last page', 'dyapress-ecommerce' ); ?></span>
				<span aria-hidden="true">‹</span>
			    </a><?php
	    } else { ?>
			    <span class="tablenav-pages-navspan button disabled" aria-hidden="true">«</span>
			    <span class="tablenav-pages-navspan button disabled" aria-hidden="true">‹</span><?php
	    } ?>
			<span class="paging-input"<?php if ( $pos != 'top' ) echo ' id="table-paging">'; else { ?>>
			    <label for="current-page-selector" class="screen-reader-text"><?php _e( 'Actual page', 'dyapress-ecommerce' ); ?></label>
			    <input class="current-page" id="current-page-selector" type="text" name="paged" value="<?php echo $paged; ?>" size="<?php echo strlen( $result->max_num_pages ); ?>" aria-describedby="table-paging"><?php } ?>
				<span class="tablenav-paging-text"><?php if ( $pos != 'top' ) echo $paged; _e( ' of ', 'dyapress-ecommerce' ); ?>
				    <span class="total-pages"><?php echo $result->max_num_pages; ?></span>
				</span>
			    </span><?php
	    if ( $paged < $result->max_num_pages ) { ?>
			    <a class="next-page button" href="<?php echo $url.'&paged='.($paged+1); ?>">
				<span class="screen-reader-text"><?php _e( 'Next page', 'dyapress-ecommerce' ); ?></span>
				<span aria-hidden="true">›</span>
			    </a>
			    <a class="last-page button" href="<?php echo $url.'&paged='.$result->max_num_pages; ?>">
				<span class="screen-reader-text"><?php _e( 'Last page', 'dyapress-ecommerce' ); ?></span>
				<span aria-hidden="true">»</span>
			    </a><?php
	    } else { ?>
			    <span class="tablenav-pages-navspan button disabled" aria-hidden="true">›</span>
			    <span class="tablenav-pages-navspan button disabled" aria-hidden="true">»</span><?php
	    } ?>
			</span><?php
	} ?>
			<br class="clear">
		    </div>
		</div>
<?php
    }

    public static function admin_orders_page() {
	if ( ! current_user_can( 'manage_options' ) ) {
	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}

	$url = dyapress_ecommerce()->admin_url();
	$limit = isset( $_GET['limit'] ) ? (int)$_GET['limit'] : 0;

	if ( empty( $limit ) ) {
	    $user = get_current_user_id();
	    $screen = get_current_screen();
	    $option = $screen->get_option( 'dyapress_ecommerce_orders_per_page', 'option' );
	    $limit = (int)get_user_meta( $user, $option, true );

	    if ( empty( $limit ) || $limit < 1 )
		$limit = (int)$screen->get_option( 'dyapress_ecommerce_orders_per_page', 'default' );
	}

	$args = array(
	    'paginate' => 2,
	    'limit' => $limit,
	);

	if ( isset( $_GET['paged'] ) )
	    $args['paged'] = (int)$_GET['paged'];
	if ( isset( $_GET['order_state'] ) )
	    $args['order_state'] = sanitize_key( $_GET['order_state'] );
	if ( isset( $_GET['orderby'] ) )
	    $args['orderby'] = sanitize_key( $_GET['orderby'] );
	else
	    $args['orderby'] = 'id';
	if ( isset( $_GET['order'] ) )
	    $args['order'] = sanitize_key( $_GET['order'] );
	if ( isset( $args['order'] ) && $args['order'] != 'asc' && $args['order'] != 'desc' )
	    $order = $args['orderby'] == 'id' || $args['orderby'] == 'date' ? 'desc' : 'asc';

	$args = apply_filters( 'dyapress_ecommerce_admin_orders_filter', $args );
	$date_format = apply_filters( 'dyapress_ecommerce_admin_date_format', 'd/m/Y H:i:s' );

	$result = dyapress_premium()->get_orders( $args );

	if ( empty( $result ) || is_wp_error( $result ) ) {
	    echo '<div class="wrap dyapress-ecommerce-admin">';
	    echo '<h1 class="wp-heading-inline">'.__( 'Orders', 'dyapress-ecommerce' ).'</h1>';
	    echo '<p class="status">'.__( 'An error occurred to fetch orders, please check your API is correctly configured.', 'dyapress-ecommerce' ).'</p>';
	    echo '</div>';
	} else {
	    $columns = dyapress_ecommerce()->admin_order_columns( $args, $url );
?>
	<div class="wrap dyapress-ecommerce-admin">
	<h1 class="wp-heading-inline"><?php echo __( 'Orders', 'dyapress-ecommerce' ); ?></h1><?php
	    // now manage the order, in any cases, the orders list may not be updated online immediately since
	    // the jobs are deferred in DyaPress Premium (and locks may be used on the orders involved)
	    if ( !empty( $_GET['action'] ) )
		$action = sanitize_key( $_GET['action'] );
	    elseif ( !empty( $_GET['action2'] ) )
		$action = sanitize_key( $_GET['action2'] );
	    if ( !empty( $action ) ) {
		if ( !empty( $_GET['orders'] ) ) {
		    $orders = self::sanitize_orders( $_GET['orders'] );

		    if ( $action == 'manage' )
			$message = self::manage_orders( $orders );
		    elseif ( $action == 'prepare' )
			$message = self::prepare_orders( $orders );
		    elseif ( $action == 'complete' )
			$message = self::complete_orders( $orders );
		    else
			$message = false;

		    if ( !empty( $message ) ) { ?>
	    <div id="message" class="updated notice is-dismissible">
		<?php echo $message; ?>
		<button type="button" class="notice-dismiss">
		    <span class="screen-reader-text"><?php _e("Dismiss this notice.", "dyapress_ecommerce"); ?></span>
		</button>
	    </div><?php
		    } elseif ( is_null( $message ) ) { ?>
	    <div id="message" class="error notice is-dismissible">
		<p><?php _e( 'An error occurred, nothing has been done on the orders!', 'dyapress-ecommerce' ); ?></p>
		<button type="button" class="notice-dismiss">
		    <span class="screen-reader-text"><?php _e("Dismiss this notice.", "dyapress_ecommerce"); ?></span>
		</button>
	    </div><?php
		    }
		} else { ?>
	    <div id="message" class="error notice is-dismissible">
		<p><?php _e( 'Please select orders to apply an action on them...', 'dyapress-ecommerce' ); ?></p>
		<button type="button" class="notice-dismiss">
		    <span class="screen-reader-text"><?php _e("Dismiss this notice.", "dyapress_ecommerce"); ?></span>
		</button>
	    </div><?php
		}
	    } ?>
	    <hr class="wp-header-end">
	    <h2 class="screen-reader-text"><?php echo __( 'Filter orders list', 'dyapress-ecommerce' ); ?></h2>
	    <ul class="subsubsub">
		<li class="all">
		    <a href="<?php echo $url; ?>"<?php if ( empty( $args['order_state'] ) ) echo ' class="current"'; ?>><?php echo __( 'All', 'dyapress-ecommerce' ); ?>
			<span class="count">(<?php echo $result->order_state->all; ?>)</span>
			    </a><?php
	    if ( !empty( $result->order_state ) && is_object( $result->order_state ) )
		foreach ( $result->order_state as $order_state => $count ) {
		    if ( $order_state != "all" ) {
			echo ' |</li><li class="dyapress_ecommerce_order_state '.$order_state.'"><a href="'.$url.'&order_state='.$order_state.'"'.( !empty( $args['order_state'] ) && $args['order_state'] == $order_state ? ' class="current"' : '' ).'>';
			echo dyapress_ecommerce()->format_order_state( $order_state );
			echo '</a><span class="count">('.$count.')</span></a>';
		    }
		} ?>
		</li>
	    </ul>
	    <form id="orders-filter" method="get"><?php
		$query = parse_url( $url, PHP_URL_QUERY );
		parse_str( $query, $array );
		foreach ( $array as $k => $v ) {
		    echo '<input type="hidden" name="'.esc_html( $k ).'" value="'.esc_html( $v ).'">';
		}
		if ( isset( $args['order_state'] ) && $args['order_state'] != "all" )
		    echo '<input type="hidden" name="order_state" value="'.$args['order_state'].'">';
		self::admin_orders_tablenav( 'top', $args, $result ); ?>
		<table class="wp-list-table widefat striped dyapress-ecommerce orders">
		    <thead>
<?php		dyapress_ecommerce()->echo_columns_header( $columns ); ?>
		    </thead>
		    <tbody id="orders">
<?php
		foreach ( $result->orders as $o ) {
		    $p = dyapress_ecommerce()->admin_order_display( $o, $date_format );

		    dyapress_ecommerce()->echo_columns_body( $p, $columns,
			'id="dyapress_ecommerce_order-'.$o->id.'" class="dyapress_ecommerce_order-'.$o->id.' dyapress_ecommerce_order"' );
		}
?>
		    </tbody>
		    <tfoot>
		    </tfoot>
		</table><?php
		self::admin_orders_tablenav( 'bottom', $args, $result ); ?>
	    </form>
	</div>
	<script>
	    jQuery( document ).ready( function($) {
		var locale = (new Intl.NumberFormat()).resolvedOptions().locale;

		$( '.dyapress-ecommerce' ).find( '.format-price' ).each( function() {
		    var price = $( this ).data( 'price' );
		    var currency = $( this ).data( 'currency' );

		    $( this ).removeClass( 'format-price' ).html( new Intl.NumberFormat(locale, { style: 'currency', currency: currency }).format( price ) );

		} );
	    } );
	</script>
<?php
	}
    }

    public static function admin_settings_page() {
	if ( ! current_user_can( 'manage_options' ) ) {
	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}

	$tabs = dyapress_ecommerce()->get_settings_tabs();

	echo '<div class="wrap dyapress-ecommerce-admin"><h1>'.__( 'Configure DyaPress Premium', 'dyapress-ecommerce' ).'</h1>';

	if ( isset($_GET['status']) && $_GET['status']=='success') { ?>
	    <div id="message" class="updated notice is-dismissible">
		<p><?php _e( 'Settings updated!', 'dyapress-ecommerce' ); ?></p>
		<button type="button" class="notice-dismiss">
		    <span class="screen-reader-text"><?php _e("Dismiss this notice.", "dyapress_ecommerce"); ?></span>
		</button>
	    </div><?php
	} 

	// allow other messages to be displayed
	do_action( 'dyapress_ecommerce_admin_settings_message' );

	// check if premium licence is active
	printf( '<p>'.esc_html__( 'Your DyaPress Premium API UUID is %s', 'dyapress-ecommerce' ).'</p>', '<strong style="display: inline-block; background: rgba(255,255,255,.5); box-shadow: inset 0 1px 2px rgba(0,0,0,.04); color: black; margin: 1px; padding: 3px 5px; border: 1px solid #ddd;">'.dyapress_premium()->premium_uuid().'</strong>' );

	if ( !dyapress_premium()->licence_level() ) {
	    printf( '<p>'.esc_html__( 'You are using the free version of DyaPress Premium, please activate it at %s for better experience!', 'dyapress-ecommerce' ).'</p>', self::premium_signup_link() );
	}

	if ( isset( $_GET['tab'] ) )
	    $active = sanitize_key( $_GET['tab'] );
	if ( !empty( $active ) &&  !isset( $tabs[$active] ) )
	    $active = null;

	echo '<h2 class="nav-tab-wrapper">';
	foreach ( $tabs as $slug => $tab ) {
	    if ( empty( $active ) )
		$active = $slug;

	    echo '<a href="'.dyapress_ecommerce()->admin_url( 'settings', $tab['slug'] ).'" class="nav-tab';
	    if ( $tab['slug'] == $active ) {
		echo ' nav-tab-active';

		$active_tab = $tab;
	    }
	    echo '">'.esc_html( $tab['title'] ).'</a>';
	}
	echo '</h2>';

	call_user_func_array( $active_tab['function'], array(dyapress_ecommerce()->admin_url( 'settings', $active_tab['slug'] ) ) );

	echo '</div>';
    }

    public static function admin_settings_tab_api( $admin_tab_url ) {
	$options = dyapress_premium()->options();
?>
	<form method="post" action="<?php echo admin_url( 'admin-post.php' ); ?>">
	    <input type="hidden" name="action" value="dyapress_ecommerce_update_options" />
	    <input type="hidden" name="dyapress_ecommerce_api_nonce" value="<?php echo wp_create_nonce( 'update_dyapress_ecommerce_api' ); ?>" />
	    <table class="form-table"><tr>
		<th scope="row"><?php _e( 'LIVE or Production', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'LIVE or Production', 'dyapress-ecommerce' ); ?></span></legend>
		    <p><?php _e( 'DyaPress UUID', 'dyapress-ecommerce' ); ?> <input type="text" name="dyapress_ecommerce_live_uuid" value="<?php echo $options['live_uuid']; ?>" size="40" /> <?php _e( 'using DOLAPIKEY', 'dyapress-ecommerce' ); ?> <input type="text" name="dyapress_ecommerce_live_dolapikey" value="<?php echo esc_html( $options['live_dolapikey'] ); ?>" size="40" /></p>
		    </fieldset><p><?php self::esc_status_message( $options['live_status'], $options['live_version'] ); ?></p></td></tr><tr>
		<th scope="row"><?php _e( 'TEST or Development', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'TEST or Development', 'dyapress-ecommerce' ); ?></span></legend>
		    <p><?php _e( 'DyaPress UUID', 'dyapress-ecommerce' ); ?> <input type="text" name="dyapress_ecommerce_test_uuid" value="<?php echo $options['test_uuid']; ?>" size="40" /> <?php _e( 'using DOLAPIKEY', 'dyapress-ecommerce' ); ?> <input type="text" name="dyapress_ecommerce_test_dolapikey" value="<?php echo esc_html( $options['test_dolapikey'] ); ?>" size="40" /></p>
		    </fieldset><p><?php self::esc_status_message( $options['test_status'], $options['test_version'] ); ?></p></td></tr><tr>
		<th scope="row"><?php _e( 'DyaPress Premium API mode', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'DyaPress mode is', 'dyapress-ecommerce' ); ?></span></legend>
		    <select name="dyapress_ecommerce_mode"><?php
			echo '<option value="test"' . selected( $options['mode'], 'test' ) . '>' . __( 'TEST', 'dyapress-ecommerce' ) . '</option>';
			echo '<option value="live"' . selected( $options['mode'], 'live' ) . '>' . __( 'LIVE', 'dyapress-ecommerce' ) . '</option>';
		    ?></select>
		</fieldset></td></tr><tr>
		<th scope="row"><?php _e( 'Country of the activities', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'Country of the activities', 'dyapress-ecommerce' ); ?></span></legend><?php
	if ( dyapress_premium()->is_active() ) { ?>
		    <select name="dyapress_ecommerce_country_code"><?php
			$country_code = $options['country_code'];
			echo '<option value="">' . __( 'Please choose a country', 'dyapress-ecommerce' ) . '</option>';
			foreach ( dyapress_premium()->get_countries() as $c )
			    echo '<option value="'.esc_html($c->code).'" '.selected( $country_code, $c->code ). '>' . esc_html($c->name) . '</option>';
		    ?></select><?php
	} else {
	    echo '<p class="dyapress-ecommerce-status">'.__( 'API is not active.', 'dyapress-ecommerce' ).'</p>';
	} ?>
		</fieldset></td></tr>
	    </table>
	    <p class="submit">
		<input type="submit" class="button-primary" value="<?php _e( 'Save Changes', 'dyapress-ecommerce' ); ?>" />
	    </p>
	</form>
<?php
    }

    private static function esc_status_message( $status, $version ) {
	$message = '<span class="status '.esc_html( $status ).'">';

	if ( $status == 'active' ) {
	    $message .= __( 'DyaPress or Dolibarr is connected and active !', 'dyapress-ecommerce' );
	} elseif ( $status == 'locked' ) {
	    $message .= __( 'DyaPress or Dolibarr is connected but locked, please enable it to use it.', 'dyapress-ecommerce' );
	} elseif ( $status == 'http_response_error' ) {
	    $message .= __( 'Invalidation HTTP response from DyaPress or Dolibarr.', 'dyapress-ecommerce' );
	} elseif ( $status == 'invalid_install' ) {
	    $message .= __( 'Invalidation DyaPress or Dolibarr installation ?', 'dyapress-ecommerce' );
	} elseif ( $status == 'invalid_apikey' ) {
	    $message .= __( 'The given DOLAPIKEY is invalid, please verify it.', 'dyapress-ecommerce' );
	} elseif ( $status == 'http_request_error' ) {
	    $message .= __( 'Invalid HTTP request, maybe a bad URL in configuration ?', 'dyapress-ecommerce' );
	} elseif ( $status == 'invalid_uuid' ) {
	    $message .= __( 'Invalid DyaPress UUID : no licence registered with this UUID.', 'dyapress-ecommerce' );
	} elseif ( $status == 'empty_dolapikey' ) {
	    $message .= __( 'Empty DOLAPIKEY: please copy DOLAPIKEY of DyaPress or Dolibarr once REST API module has been activated!', 'dyapress-ecommerce' );
	} elseif ( $status == 'empty_uuid' ) {
	    $message .= sprintf( __( 'Empty DyaPress UUID: please copy UUID of DyaPress! If you are using Dolibarr please register your Dolibarr installation on %s !', 'dyapress-ecommerce' ), self::premium_signup_link( __( 'DyaPress website', 'dyapress-ecommerce' ) ) );
	} else {
	    $message .= __( 'Oops ! unknown error message !', 'dyapress-ecommerce' );
	}

	if ( !empty( $version) ) {
	    $message .= '&nbsp;'.sprintf( __( 'The version of the REST API in use is %s.', 'dyapress-ecommerce'), '<strong>'.esc_html( $version ).'</strong>' );
	}

	$message .= '</span>';

	echo $message;
    }

    private static function sanitize_orders( $orders ) {
	if ( isset( $orders ) && is_array( $orders ) )
	    return $orders;
	else
	    return null;
    }

    public static function admin_settings_tab_stripe( $admin_tab_url ) {
	$message = false;

	// return from Stripe Connect
	if ( isset( $_GET['code'] ) && isset( $_GET['state'] ) ) {
	    if ( wp_verify_nonce( $_GET['state'], 'stripe_connect' ) ) {
		if ( dyapress_premium()->stripe_connect( $_GET['code'] ) ) {
		    $message = __( 'Stripe is now connected with DyaPress Premium!', 'dyapress-ecommerce' );
		} else
		    $message = __( 'An error occurred to connect Stripe with DyaPress Premium.', 'dyapress-ecommerce' );
	    } else
		wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}

	if ( !empty( $message ) ) { ?>
   <div id="message" class="updated notice is-dismissible">
      <p><?php echo $message; ?></p>
      <button type="button" class="notice-dismiss">
         <span class="screen-reader-text"><?php _e("Dismiss this notice.", "dyapress_ecommerce"); ?></span>
      </button>
   </div>
	<?php }

	if ( !empty( dyapress_premium()->stripe_account_id() ) ) {
	    printf( '<p><strong class="dyapress-ecommerce-status active">'.esc_html__( 'Your Stripe account is connected with DyaPress Premium', 'dyapress-ecommerce' ).'</strong></p>' ); 
	} elseif ( empty( dyapress_premium()->stripe_public_key() ) ) {
	    printf( '<p>'.esc_html__( 'Connect DyaPress Premium with your Stripe account to accept payment made by card on your site: %s', 'dyapress-ecommerce' ).'</p>', 
		'<p><a href="https://connect.stripe.com/oauth/authorize?response_type=code&client_id='.dyapress_premium()->stripe_application_id().'&scope=read_write&state='.dyapress_premium()->stripe_connect_state($admin_tab_url, wp_create_nonce( 'stripe_connect' ) ).'" class="dyapress-ecommerce-stripe-connect"><span>'.__( 'Connect with Stripe', 'dyapress-ecommerce' ).'</span></a>' );
	}
?>
	<form method="post" action="<?php echo admin_url( 'admin-post.php' ); ?>">
	    <input type="hidden" name="action" value="dyapress_ecommerce_update_options" />
	    <input type="hidden" name="dyapress_ecommerce_stripe_nonce" value="<?php echo wp_create_nonce( 'update_dyapress_ecommerce_stripe' ); ?>" />
	    <table class="form-table"><tr>
		<th scope="row"><?php _e( 'Stripe payment type', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'Stripe payment type', 'dyapress-ecommerce' ); ?></span></legend><?php
	if ( dyapress_premium()->is_active() ) { ?>
		    <select name="dyapress_ecommerce_stripe_payment_type_id"><?php
			echo '<option value="0">'.__( 'Please choose the payment type of Stripe', 'dyapress-ecommerce' ).'</option>';
			$stripe_payment_type_id = dyapress_ecommerce()->stripe_payment_type_id();
			foreach ( dyapress_premium()->get_payment_types() as $p )
			    echo '<option value="'.esc_html($p->id).'" '.selected( $stripe_payment_type_id, $p->id ). '>' . esc_html($p->name) . '</option>';
		    ?></select><?php
	} else {
	    echo '<p class="dyapress-ecommerce-status">'.__( 'API is not active.', 'dyapress-ecommerce' ).'</p>';
	} ?>
		</fieldset></td></tr>
		<th scope="row"><?php _e( 'Stripe payment account', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'Stripe payment account', 'dyapress-ecommerce' ); ?></span></legend><?php
	if ( dyapress_premium()->is_active() ) { ?>
		    <select name="dyapress_ecommerce_stripe_payment_account_id"><?php
			echo '<option value="0">'.__( 'Please choose the payment account of Stripe', 'dyapress-ecommerce' ).'</option>';
			$stripe_payment_account_id = dyapress_ecommerce()->stripe_payment_account_id();
			foreach ( dyapress_premium()->get_bank_accounts() as $p )
			    echo '<option value="'.esc_html($p->id).'" '.selected( $stripe_payment_account_id, $p->id ). '>' . esc_html($p->name) . '</option>';
		    ?></select><?php
	} else {
	    echo '<p class="dyapress-ecommerce-status">'.__( 'API is not active.', 'dyapress-ecommerce' ).'</p>';
	} ?>
		</fieldset></td></tr>
	    </table>
	    <p class="submit">
		<input type="submit" class="button-primary" value="<?php _e( 'Save Changes', 'dyapress-ecommerce' ); ?>" />
	    </p>
	</form><?php
    }

    public static function admin_settings_tab_account( $admin_tab_url ) {
	$options = get_option( 'dyapress_ecommerce_options' );
?>
	<form method="post" action="<?php echo admin_url( 'admin-post.php' ); ?>">
	    <input type="hidden" name="action" value="dyapress_ecommerce_update_options" />
	    <input type="hidden" name="dyapress_ecommerce_account_nonce" value="<?php echo wp_create_nonce( 'update_dyapress_ecommerce_account' ); ?>" />
	    <table class="form-table"><tr>

		<tr><th scope="row"><?php _e( 'My account page', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'My account page', 'dyapress-ecommerce' ); ?></span></legend>
		    <?php wp_dropdown_pages( array(
			'name' => 'dyapress_ecommerce_myaccount_page_id',
			'selected' => !empty( $options['myaccount_page_id'] ) ? (int)$options['myaccount_page_id'] : 0,
			'show_option_none' => __( 'Select a page&hellip;', 'dyapress-ecommerce' ),
		    ) ); ?>
		</fieldset></td></tr><tr>
		<th scope="row"><?php _e( 'Account sections', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'DyaPress account sections', 'dyapress-ecommerce' ); ?></span></legend>
		    <p>
			<input name="dyapress_ecommerce_display_account_proposals" type="checkbox" value="1"<?php echo !empty( $options['display_account_proposals'] ) ? ' checked="checked"' : ''; ?> />
			<?php _e( 'Display DyaPress proposals associated to the user ?', 'dyapress-ecommerce' ); ?>
		    </p>
		    <p>
			<input name="dyapress_ecommerce_display_account_invoices" type="checkbox" value="1"<?php echo !empty( $options['display_account_invoices'] ) ? ' checked="checked"' : ''; ?> />
			<?php _e( 'Display DyaPress invoices associated to the user ?', 'dyapress-ecommerce' ); ?>
		    </p>
		</fieldset></td></tr><tr>
		<th scope="row"><?php _e( 'Send mail from', 'dyapress-ecommerce' ); ?></th>
		<td><fieldset>
		    <legend class="screen-reader-text"><span><?php _e( 'DyaPress mail from', 'dyapress-ecommerce' ); ?></span></legend>
		    <p><input type="text" name="dyapress_ecommerce_mail_from_name" value="<?php echo !empty( $options['mail_from_name'] ) ? $options['mail_from_name'] : ''; ?>" /> <?php _e( 'Mail sender name', 'dyapress-ecommerce' ); ?></p>
		    <p><input type="text" name="dyapress_ecommerce_mail_from_email" value="<?php echo !empty( $options['mail_from_email'] ) ? $options['mail_from_email'] : ''; ?>" /> <?php _e( 'Mail sender address', 'dyapress-ecommerce' ); ?></p>
		</fieldset></td></tr>
	    </table>
	    <p class="submit">
		<input type="submit" class="button-primary" value="<?php _e( 'Save Changes', 'dyapress-ecommerce' ); ?>" />
	    </p>
	</form>
</div><?php
    }

    static function update_options() {
	if ( isset( $_POST['dyapress_ecommerce_api_nonce'] ) && wp_verify_nonce( $_POST['dyapress_ecommerce_api_nonce'], 'update_dyapress_ecommerce_api' ) ) {
	    $redirect_url = dyapress_ecommerce()->admin_url( 'settings' );
	    $options = array();

	    if ( isset( $_POST['dyapress_ecommerce_live_uuid'] ) )
		$options['live_uuid'] = sanitize_key( $_POST['dyapress_ecommerce_live_uuid'] );
	    if ( isset( $_POST['dyapress_ecommerce_live_dolapikey'] ) )
		$options['live_dolapikey'] = sanitize_key( $_POST['dyapress_ecommerce_live_dolapikey'] );
	    if ( isset( $_POST['dyapress_ecommerce_test_uuid'] ) )
		$options['test_uuid'] = sanitize_key( $_POST['dyapress_ecommerce_test_uuid'] );
	    if ( isset( $_POST['dyapress_ecommerce_test_dolapikey'] ) )
		$options['test_dolapikey'] = sanitize_key( $_POST['dyapress_ecommerce_test_dolapikey'] );
	    if ( isset( $_POST['dyapress_ecommerce_mode'] ) )
		$options['mode'] = sanitize_key( $_POST['dyapress_ecommerce_mode'] );
	    if ( isset( $_POST['dyapress_ecommerce_country_code'] ) )
		$options['country_code'] = sanitize_key( $_POST['dyapress_ecommerce_country_code'] );

	    // FIXME: options are always validated before saving
	    if ( !empty( dyapress_premium()->options( $options ) ) )
		$redirect_url .= '&status=success';
	    else
		$redirect_url .= '&status=failed';
	} elseif ( isset( $_POST['dyapress_ecommerce_stripe_nonce'] ) && wp_verify_nonce( $_POST['dyapress_ecommerce_stripe_nonce'], 'update_dyapress_ecommerce_stripe' ) ) {
	    $redirect_url = dyapress_ecommerce()->admin_url( 'settings', 'stripe' );
	    $options = array();

	    if ( isset( $_POST['dyapress_ecommerce_stripe_payment_type_id'] ) )
		$options['stripe_payment_type_id'] = (int)$_POST['dyapress_ecommerce_stripe_payment_type_id'];
	    if ( isset( $_POST['dyapress_ecommerce_stripe_payment_account_id'] ) )
		$options['stripe_payment_account_id'] = (int)$_POST['dyapress_ecommerce_stripe_payment_account_id'];

	    // FIXME: options are always validated before saving
	    if ( !empty( dyapress_ecommerce()->options( $options ) ) )
		$redirect_url .= '&status=success';
	    else
		$redirect_url .= '&status=failed';
	} elseif ( isset( $_POST['dyapress_ecommerce_account_nonce'] ) && wp_verify_nonce( $_POST['dyapress_ecommerce_account_nonce'], 'update_dyapress_ecommerce_account' ) ) {
	    $redirect_url = dyapress_ecommerce()->admin_url( 'settings', 'account' );
	    $options = array();

	    $options['display_account_proposals'] = (int)$_POST['dyapress_ecommerce_display_account_proposals'];
	    $options['display_account_invoices'] = (int)$_POST['dyapress_ecommerce_display_account_invoices'];
	    if ( isset( $_POST['dyapress_ecommerce_myaccount_page_id'] ) )
		$options['myaccount_page_id'] = (int)$_POST['dyapress_ecommerce_myaccount_page_id'];
	    if ( isset( $_POST['dyapress_ecommerce_mail_from_name'] ) )
		$options['mail_from_name'] = sanitize_text_field( $_POST['dyapress_ecommerce_mail_from_name'] );
	    if ( isset( $_POST['dyapress_ecommerce_mail_from_email'] ) )
		$options['mail_from_email'] = sanitize_email( $_POST['dyapress_ecommerce_mail_from_email'] );

	    // FIXME: options are always validated before saving
	    if ( !empty( dyapress_ecommerce()->options( $options ) ) )
		$redirect_url .= '&status=success';
	    else
		$redirect_url .= '&status=failed';
	}

	if ( isset( $redirect_url ) ) {
	    header( 'Location: '.$redirect_url );
	    exit;
	} else
	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
    }

    /**
     * Retrieves DyaPress Premium signup URL according to some locale.
     */
    static function premium_signup_link( $text = null ) {
	if ( preg_match( '/^fr_/', get_locale() ) )
	    $url = 'https://dyapress.fr/inscription';
	else
	    $url = 'https://dyapress.com/signup';

	if ( empty( $text ) )
	    $text = $url;

	$options = dyapress_premium()->options();

	if ( is_array( $options ) && !empty( $options['auth'] ) )
	    return '<a href="'.esc_url( $url.'?auth='.urlencode( $options['auth'] ) ).'" target="_blank">'.$text.'</a>';
	else
	    return '<a href="'.esc_url( $url ).'" target="_blank">'.$text.'</a>';
    }
}

