<?php
/**
 * DyaPress e-Commerce Download.
 *
 * @class    DyaPressECommerceDownload
 * @package  DyaPressECommerce/Classes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * WordPress DyaPress e-Commerce plugin :
 * Copyright (C) 2018-2020      François Pons        <fpons@aytechnet.fr>
 *
 * Please see https://dyapress.com/
 */

defined( 'ABSPATH' ) or die( 'Oops' );

if ( !class_exists( 'DyaPressECommerceDownload' ) ) {
    class DyaPressECommerceDownload {
	private static $locale;

	public static function init() {
	    if ( isset( $_GET['locale'] ) )
		self::$locale = sanitize_key( $_GET['locale'] );
	    else
		self::$locale = get_locale();

	    if ( isset( $_GET['dyapress_download_order'] ) && (int)$_GET['dyapress_download_order'] > 0 )
		self::download_order( (int)$_GET['dyapress_download_order'] );
	    elseif ( isset( $_GET['dyapress_download_order_invoice'] ) && (int)$_GET['dyapress_download_order_invoice'] > 0 )
		self::download_order_invoice( (int)$_GET['dyapress_download_order_invoice'] );
	    elseif ( isset( $_GET['dyapress_download_proposal'] ) && (int)$_GET['dyapress_download_proposal'] > 0 )
		self::download_proposal( (int)$_GET['dyapress_download_proposal'] );
	    elseif ( isset( $_GET['dyapress_download_invoice'] ) && (int)$_GET['dyapress_download_invoice'] > 0 )
		self::download_invoice( (int)$_GET['dyapress_download_invoice'] );
	}

	private static function download_document( $document, $filename, $template = null ) {
	    $doc = dyapress_premium()->download_document( $document );
	    if ( empty( $doc ) )
		$doc = dyapress_premium()->build_document(
		    $document,
		    apply_filters( 'dyapress_download_document_template', $template, $document ),
		    apply_filters( 'dyapress_download_document_language', self::$locale, $document )
		);

	    if ( !empty( $doc ) ) {
		if ( ! defined( 'DONOTCACHEPAGE' ) ) {
		    define( 'DONOTCACHEPAGE', true );
		}

		header('Content-Type: application/pdf');
		header('Content-Disposition: inline; filename="'.$filename.'"');

		print $doc;

		die();
	    } else {
		wp_die( __( 'No file is available.', 'dyapress-ecommerce' ));
	    }
	}

	public static function download_order( $id ) {
	    // check if a user is connected
	    if ( $user = dyapress_premium()->get_current_user() ) {
		// check if invoice exists
		if ( $order = dyapress_premium()->get_order( $id ) ) {
		    // check if invoice is available for download
		    if ( current_user_can( 'manage_options' ) || isset( $user['thirdparties'][$order->thirdparty_id] ) ) {
			self::download_document( 'commande/'.$order->order_reference.'/'.$order->order_reference.'.pdf', $order->order_reference.'.pdf' );
		    }
		}
	    }

	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}

	public static function download_order_invoice( $id ) {
	    // check if a user is connected
	    if ( $user = dyapress_premium()->get_current_user() ) {
		// check if invoice exists
		if ( $order = dyapress_premium()->get_order( $id ) ) {
		    // check if invoice is available for download
		    if ( current_user_can( 'manage_options' ) || isset( $user['thirdparties'][$order->thirdparty_id] ) ) {
			self::download_document( 'facture/'.$order->order_invoice_reference.'/'.$order->order_invoice_reference.'.pdf', $order->order_invoice_reference.'.pdf' );
		    }
		}
	    }

	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}

	public static function download_proposal( $id ) {
	    // check if a user is connected
	    if ( $user = dyapress_premium()->get_current_user() ) {
		// check if proposal exists
		if ( $proposal = dyapress_premium()->get_proposal( $id ) ) {
		    // check if proposal is available for download
		    if ( current_user_can( 'manage_options' ) || isset( $user['thirdparties'][$proposal->thirdparty_id] ) ) {
			self::download_document( $proposal->document, $proposal->reference.".pdf", !empty( $proposal->template ) ? $proposal->template : null );
		    }
		}
	    }

	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}
	public static function download_invoice( $id ) {
	    // check if a user is connected
	    if ( $user = dyapress_premium()->get_current_user() ) {
		// check if invoice exists
		if ( $invoice = dyapress_premium()->get_invoice( $id ) ) {
		    // check if invoice is available for download
		    if ( current_user_can( 'manage_options' ) || isset( $user['thirdparties'][$invoice->thirdparty_id] ) ) {
			self::download_document( $invoice->document, $invoice->reference.".pdf", !empty( $invoice->template ) ? $invoice->template : null );
		    }
		}
	    }

	    wp_die( __( 'You do not have rights to access this page.', 'dyapress-ecommerce' ));
	}
    }

    DyaPressECommerceDownload::init();
}
