<?php
/**
 * DyaPress e-Commerce Shortcodes.
 *
 * @class    DyaPressECommerceShortcodes
 * @package  DyaPressECommerce/Classes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * WordPress DyaPress e-Commerce plugin :
 * Copyright (C) 2018-2020      François Pons        <fpons@aytechnet.fr>
 *
 * Please see https://dyapress.com/
 */

defined( 'ABSPATH' ) or die( 'Oops' );

if ( !class_exists( 'DyaPressECommerceShortcodes' ) ) {
    class DyaPressECommerceShortcodes {
	public static function init() {
	    $shortcodes = array(
		'registration',
		'registration_purchase',
		'myaccount',
	    );

	    foreach ($shortcodes as $name ) {
		add_shortcode( 'dyapress_ecommerce_' . $name, __CLASS__ . '::' . $name );
	    }
	}

	public static function registration( $atts ) {
	    extract( shortcode_atts( array(
		'tpl' => '',
		'button_class' => 'btn btn-default',
		'button_label' => __( 'Create account', 'dyapress-ecommerce' ),
	    ), $atts ) );

	    if ( dyapress_premium()->is_available() ) {
		if ( is_user_logged_in() ) {
		    $result = '<div class="dyapress-ecommerce dyapress-ecommerce-registration">
			<p class="important">
			    '.sprintf( __( 'Your are already logged in, please <a href="%s">logout</a> to register a new account.', 'dyapress-ecommerce' ), esc_url( wp_logout_url( get_permalink() ) ) ).'
			</p>
		    </form>';
		} else {
		    $result = '<div class="dyapress-ecommerce dyapress-ecommerce-registration"><form>
			<input type="hidden" name="dyapress-ecommerce_nonce" value="'.wp_create_nonce( 'registration' ).'">';

		    $result .= dyapress_ecommerce()->get_template_part( 'registration', $tpl, $atts );
		    $result .= '<button type="submit" class="'.esc_html( $button_class ).'">'.esc_html( $button_label ).'</button>';

		    $result .= '</form><div class="messages"></div></div>';
		}

		return $result;
	    } else {
		return __( 'Registration is currently not available, please try again later...', 'dyapress-ecommerce' );
	    }
	}

	public static function registration_purchase( $atts ) {
	    extract( shortcode_atts( array(
		'tpl' => '',
		'payment' => 'stripe',
		'order_name' => '',
	    ), $atts ) );

	    if ( dyapress_premium()->is_available() ) {
		$result = '<div class="dyapress-ecommerce dyapress-ecommerce-prices dyapress-ecommerce-registration-purchase">
		    <form data-is_vat_applicable="'.dyapress_premium()->is_vat_applicable().'">
		      <input type="hidden" name="dyapress-ecommerce_nonce" value="'.wp_create_nonce( 'registration_purchase' ).'">';

		if ( $order_name )
		    $result .= '<input type="hidden" name="dyapress_ecommerce_order_name" value="'.esc_html( $order_name ).'">';

		$result .= dyapress_ecommerce()->get_template_part( 'products_table', $tpl, $atts );
		$result .= dyapress_ecommerce()->get_template_part( 'registration', $tpl, $atts );
		if ( $payment == 'stripe' )
		    $result .= dyapress_ecommerce()->get_template_part( 'stripe_payment', $tpl, $atts );
		else
		    $result .= dyapress_ecommerce()->get_template_part( 'no_payment', $tpl, $atts );

		$result .= '</form><div class="messages"></div></div>';

		return $result;
	    } else {
		return __( 'Purchase is currently not available, please try again later...', 'dyapress-ecommerce' );
	    }
	}

	public static function myaccount( $atts ) {
	    extract( shortcode_atts( array(
		'tpl' => '',
	    ), $atts ) );

	    $result = '';

	    if ( isset($_GET['activate']) ) {
		$result .= self::_activate_account( sanitize_text_field( $_GET['activate'] ) );
	    } else {
		if ( is_user_logged_in() )
		    $result .= dyapress_ecommerce()->get_template_part( 'account/myaccount', $tpl, $atts );
		else
		    $result .= dyapress_ecommerce()->get_template_part( 'account/login', $tpl, $atts );
	    }

	    return $result;
	}

	private static function _activate_account( $token ) {
	    $result = '';

	    if ( $user = dyapress_ecommerce()->verify_myaccount_activation_token( $token ) ) {
		// activate token is valid, so check for existing user by examining return value of get_user() :
		//   this is the current_user if everything match
		//   or false if the user does not exists
		//   or null if the user already exists but with different data (firstname or lastname or ...)
		$validated_user = dyapress_premium()->validate_user( $user );
		$existing_user = dyapress_premium()->get_user( $validated_user );

		if ( isset( $existing_user ) ) {
		    if ( $existing_user == false ) {
			// the customer doesn't exists
			$result .= '<p class="important">'.__( 'Thanks for activating your account', 'dyapress-ecommerce' ).'</p>';

			// FIXME: search_thirdparty() first, display the choices with creating a new user,
			// except if the no thirdparty is found or is the same of the user (in such case, create it directly)
			// and ask for the password
			$existing_user = dyapress_premium()->create_thirdparty( $validated_user, 'customer' );

			if ( isset( $user['order_id'] ) && $user['order_id'] > 0 ) {
			    if ( dyapress_premium()->confirm_order( $user['order_id'], $created_user ) ) {
				$result .= '<p class="important">' .
				    sprintf( __( 'Your order #%d is confirmed.', 'dyapress-ecommerce'), $user['order_id'] ).
				    '</p>';
			    } else {
				$result .= '<p class="error">' .
				    sprintf( __( 'Your order #%d cannot be confirmed.', 'dyapress-ecommerce'), $user['order_id'] ).
				    '</p>';
			    }
			}
		    } else {
			// the user already exists
			if ( empty( $existing_user->thirdparty_id ) ) {
			    // but no thridparty have been created, this will happen for already existing user when installing DyaPress Premium
			    $validated_user->ID = $existing_user->ID;

			    $existing_user = dyapress_premium()->update_thirdparty( $validated_user );

			    $result .= '<p class="important">'.__( 'Your account has been activated', 'dyapress-ecommerce' ).'</p>';
			} else {
			    $result .= '<p class="important">'.__( 'Your account has already been activated', 'dyapress-ecommerce' ).'</p>';
			}

			if ( isset( $user['order_id'] ) && $user['order_id'] > 0 ) {
			    if ( dyapress_premium()->confirm_order( $user['order_id'], $existing_user ) ) {
				$result .= '<p class="important">' .
				    sprintf( __( 'Your order #%d is confirmed.', 'dyapress-ecommerce'), $user['order_id'] ).
				    '</p>';
			    } else {
				$result .= '<p class="error">' .
				    sprintf( __( 'Your order #%d cannot be confirmed.', 'dyapress-ecommerce'), $user['order_id'] ).
				    '</p>';
			    }
			}
		    }

		    $result .= '<div class="dyapress-ecommerce dyapress-ecommerce-password"><form>
			<input type="hidden" name="dyapress-ecommerce_nonce" value="'.wp_create_nonce( 'reset_password' ).'">
			<input type="hidden" name="dyapress-ecommerce_user_id" value="'.$existing_user->ID.'">
			<input type="hidden" name="dyapress-ecommerce_email" value="'.esc_attr( $existing_user->email ).'">';
		    $result .= dyapress_ecommerce()->get_template_part( 'reset_password' );
		    $result .= '<button type="submit" class="btn btn-default btn-reset-passwrd">'.__( 'Reset password', 'dyapress-ecommerce' ).'</button></form><div class="messages"></div></div>';

		    return $result;
		} else {
		    $result = '<p class="error">' .
			__( 'The account you try to activate cannot be activated ! A different account with the same email already exists.', 'dyapress-ecommerce' ).
			'</p>';

		    if ( isset( $user['order_id'] ) && $user['order_id'] > 0 ) {
			// This is better to send a mail to shop holder to fix the order ?
			$result .= '<p class="error">' .
			    sprintf( __( 'Please contact us to finalize the associated order #%d !', 'dyapress-ecommerce'), $user['order_id'] ).
			    '</p>';
		    }
		}
	    } else {
		$result .= '<p class="error">' . __( 'Your activation link is invalid', 'dyapress-ecommerce' ) . '</p>';
	    }

	    return $result;
	}
    }

    DyaPressECommerceShortcodes::init();
}
