( function( $ ) {

    'use strict';

    if ( typeof dyapress_ecommerce === 'undefined' || dyapress_ecommerce === null ) {
	return;
    }

    $.fn.format_price = function( price ) {
	if ( typeof dyapress_ecommerce.locale == 'undefined' )
	    dyapress_ecommerce.locale = (new Intl.NumberFormat()).resolvedOptions().locale;

	this.data( 'price', price );

	var currency = this.data( 'currency' );
	if ( typeof currency == 'undefined' || currency == '' )
	    currency = dyapress_ecommerce.default_currency;

	this.html( new Intl.NumberFormat(dyapress_ecommerce.locale, { style: 'currency', currency: currency }).format( price ) );

	return this;
    }

    $.fn.format_prices = function() {
	if ( typeof dyapress_ecommerce.locale == 'undefined' )
	    dyapress_ecommerce.locale = (new Intl.NumberFormat()).resolvedOptions().locale

	this.find( '.format-price' ).each( function() {
	    var price = $( this ).data( 'price' );
	    var currency = $( this ).data( 'currency' );
	    if ( typeof currency == 'undefined' || currency == '' )
		currency = dyapress_ecommerce.default_currency;

	    $( this ).removeClass( 'format-price' ).html( new Intl.NumberFormat(dyapress_ecommerce.locale, { style: 'currency', currency: currency }).format( price ) );

	} );

	return this;
    }

    dyapress_ecommerce.update_form_prices = function( form ) {
	var is_vat_applicable = form.data('is_vat_applicable');
	var total_price_vat_excl = 0.0;
	var total_price_vat_incl = 0.0;

	form.find('.dyapress-ecommerce_product_quantity' ).each( function() {
	    var product_id = parseInt( $(this).attr( 'name' ).replace( /dyapress-ecommerce_product_quantity\[(\d+)\]/, '$1' ) );
	    var product_quantity = parseInt( $(this).val() );
	    var product_infos = $(this).data('product_infos');

	    var total_product_price = product_infos.product_price * product_quantity;

	    form.find("span.total-price-vat-excl__" + product_id + " span.price").format_price(total_product_price);
	    total_price_vat_excl += total_product_price;
	    if ( is_vat_applicable )
		total_price_vat_incl += total_product_price*(100+product_infos.product_vat)/100;
	} );

	if ( !is_vat_applicable )
	    total_price_vat_incl = total_price_vat_excl;

	form.find('.total-vat .price').format_price(total_price_vat_incl-total_price_vat_excl);
	form.find('.total-price-vat-excl .price').format_price(total_price_vat_excl);
	form.find('.total-price-vat-incl .price').format_price(total_price_vat_incl);
    }

    dyapress_ecommerce.update_product_quantity = function( input ) {
	var product_id = parseInt( input.attr( 'name' ).replace( /dyapress-ecommerce_product_quantity\[(\d+)\]/, '$1' ) );
	var product_quantity = parseInt( input.val() );
	var form = input.parents( 'form');

	form.find('.prices').hide();

	$.post( dyapress_ecommerce.ajax_url, {
	    action: 'dyapress_get_product_infos',
	    product_id: product_id,
	    product_quantity: product_quantity,
	}, function(response) {
	    var product_infos = JSON.parse(response);
	    input.data('product_infos', product_infos );

	    form.find("span.unit-price-vat-excl__"+product_id+" span.price").format_price(product_infos.product_price);

	    dyapress_ecommerce.update_form_prices( form );

	    form.find('.prices').show();
	} );
    }

    dyapress_ecommerce.update_is_vat_applicable = function( form ) {
	var is_vat_applicable = form.data('is_vat_applicable');
	var country_id = parseInt( form.find('.dyapress-ecommerce_country').val() );
	var vat_number = form.find('.dyapress-ecommerce_vat_number').val();

	$.post( dyapress_ecommerce.ajax_url, {
	    action: 'dyapress_is_vat_applicable',
	    country_id: country_id,
	    vat_number: vat_number
	}, function(response) {

	    if ( is_vat_applicable != parseInt( response ) ) {
		form.find('.prices').hide();

		is_vat_applicable = parseInt( response );
		if ( is_vat_applicable )
		    form.find('.use-vat').show();
		else
		    form.find('.use-vat').hide();

		form.data('is_vat_applicable', is_vat_applicable );

		dyapress_ecommerce.update_form_prices( form );

		form.find('.prices').show();
	    }
	} );
    }

    dyapress_ecommerce.refresh_visible_sections = function() {
	$( '#dyapress-ecommerce-account .ui-tabs-panel[aria-hidden="false]' ).each(function(){
	    var panel = $( this );
	    var name = panel.attr( 'id' ).replace( 'tab-', '' );

	    if ( name ) {
		panel.hide();
		$.post( dyapress_ecommerce.ajax_url, {
		    action: 'dyapress_get_account_section',
		    name: name,
		}, function(response) {
		    var result = JSON.parse(response);
		    panel.html(result).format_prices();
		    dyapress_ecommerce.account_init(); // FIXME: redo account initialization
		    panel.show();
		} );
	    }
	} );
    }

    dyapress_ecommerce.registration_purchase = function( form, param ) {
	form.fadeOut();
	form.parent().find('.messages').html('<div class="info">' + dyapress_ecommerce.messages.processing_order + '</div>');

	$.post( dyapress_ecommerce.ajax_url, {
	    action: 'dyapress_registration_purchase',
	    param: param
	}, function(response) {
	    var result = JSON.parse(response);

	    if (result.success) {
		if (typeof result.payment != 'undefined' && result.payment.requires_confirmation) {
		    dyapress_ecommerce.stripe.handleCardAction(result.payment.stripe_payment_intent_client_secret).then(function(res){
			if (res.error) {
			    // restore form to allow trying another purchase
			    form.find( '.stripe .card-errors' ).html( dyapress_ecommerce.messages.card_action_failed ).format_prices();
			    form.find( 'button[type="submit"]' ).removeProp( "disabled" );
			    form.parent().find('.messages').html('');
			    form.fadeIn();
			} else {
			    // card actions where successful so let propagate it, nonce is already created in result
			    $.post( dyapress_ecommerce.ajax_url, {
				action: 'dyapress_confirm_purchase',
				param: result
			    }, function(resp) {
				var r = JSON.parse(resp);

				form.parent().find('.messages').html( r.message ).format_prices();
			    } );
			}
		    } );
		} else {
		    form.parent().find('.messages').html( result.message ).format_prices();
		}
	    } else {
		form.parent().find('.messages').html( result.message ).format_prices();
	    }
	} );
    }

    dyapress_ecommerce.account_init = function() {
	$( '.dyapress-ecommerce-account-update form' ).submit( function( event ) {
	    event.preventDefault();
	    var form = $( this );

	    form.fadeOut();
	    form.parent().find('.messages').html('<div class="info">' + dyapress_ecommerce.messages.processing + '</div>');

	    var param = new Object();
	    $( this ).find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each( function() {
		param[ $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' ) ] = $( this ).val();
	    } );

	    $.post( dyapress_ecommerce.ajax_url, {
		action: 'dyapress_account_update',
		param: param
	    }, function(response) {
		var result = JSON.parse(response);
		var html = '<div class="' + (result.success ? 'success' : 'failed') + '">' + result.message + '</div>';

		form.fadeIn();
		form.parent().find('.messages').html(html).format_prices();
	    } );
	} );

	$( '.dyapress-ecommerce-thirdparties .actions a' ).click( function( event ) {
	    event.preventDefault();
	    var thirdparty_id = parseInt( $( this ).data("thirdparty_id") );
	    var form = $( this ).parents( '.dyapress-ecommerce-thirdparties' ).find( 'form' );

	    form.parent().find( '.list' ).fadeOut();

	    form.find( 'h3' ).html( dyapress_ecommerce.messages.update_thirdparty );
	    form.find( 'button' ).html( dyapress_ecommerce.messages.update_thirdparty );
	    form.find( 'input[name="dyapress-ecommerce_thirdparty_id"]' ).val( thirdparty_id );

	    if ( $( this ).attr( 'href' ) == '#delete' ) {
		alert( "deleting thirdparty where id = "+thirdparty_id);
	    } else if ( typeof dyapress_ecommerce.current_user.thirdparties[thirdparty_id] != 'undefined' ) {
		var thirdparty = dyapress_ecommerce.current_user.thirdparties[thirdparty_id];

		form.find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each(function() {
		    var name = $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' );
		    if ( typeof thirdparty[name] != 'undefined' ) {
			$( this ).val( thirdparty[name] );
		    }
		});
	    }
	} );

	$( '.dyapress-ecommerce-thirdparties form' ).submit( function( event ) {
	    event.preventDefault();
	    var form = $( this );

	    form.fadeOut();
	    form.parent().find('.messages').html('<div class="info">' + dyapress_ecommerce.messages.processing + '</div>');

	    var param = new Object();
	    $( this ).find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each( function() {
		param[ $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' ) ] = $( this ).val();
	    } );

	    $.post( dyapress_ecommerce.ajax_url, {
		action: 'dyapress_thirdparty_update',
		param: param
	    }, function(response) {
		var result = JSON.parse(response);
		var html = '<div class="' + (result.success ? 'success' : 'failed') + '">' + result.message + '</div>';

		if ( typeof result.current_user != 'undefined' )
		    dyapress_ecommerce.current_user = result.current_user;

		// FIXME: simpler to fully refresh current section
		dyapress_ecommerce.refresh_visible_sections();
	    } );
	} );
    }

    $( function() {
	dyapress_ecommerce.account_init();

	$( '.dyapress-ecommerce-prices' ).format_prices();

	$( '.dyapress-ecommerce-registration-purchase form' ).each( function() {
	    var is_vat_applicable = $(this).data('is_vat_applicable');

	    if ( typeof is_vat_applicable == "undefined" ) {
		var country_id = parseInt( $(this).find('.dyapress-ecommerce_country').val() );
		var vat = $(this).find('.dyapress-ecommerce_vat_number').val();

		$.post( dyapress_ecommerce.ajax_url, {
		    action: 'dyapress_is_vat_applicable',
		    country_id: country_id,
		    vat: vat
		}, function(response) {
		    is_vat_applicable = parseInt( response );
		    form.data('is_vat_applicable', is_vat_applicable );
		} );
	    }
	} );
	$( '.dyapress-ecommerce-registration-purchase form .dyapress-ecommerce_product_quantity' ).each( function() {
	    dyapress_ecommerce.update_product_quantity( $(this) );
	} );

	$( '.dyapress-ecommerce form .thirdparty' ).change( function( event ) {
	    var form = $(this).parents( 'form' );
	    var thirdparty_id = parseInt( $( this ).val() );

	    if ( thirdparty_id == 0  ) {
		form.find( '.thirdparty input[name^="dyapress-ecommerce_"], .thirdparty select[name^="dyapress-ecommerce_"], .thirdparty textarea[name^="dyapress-ecommerce_"]' ).each(function() {
		    $( this ).removeProp( 'disabled' );
		    $( this ).val( '' );
		} );
	    } else if ( dyapress_ecommerce.current_user != null && typeof dyapress_ecommerce.current_user.thirdparties[thirdparty_id] != 'undefined' ) {
		var current_thirdparty = dyapress_ecommerce.current_user.thirdparties[thirdparty_id];

		form.find( '.thirdparty input[name^="dyapress-ecommerce_"], .thirdparty select[name^="dyapress-ecommerce_"], .thirdparty textarea[name^="dyapress-ecommerce_"]' ).each(function() {
		    var name = $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' );
		    if ( typeof current_thirdparty[name] != 'undefined' ) {
			$( this ).prop( 'disabled', 'disabled' );
			$( this ).val( current_thirdparty[name] );
		    }
		});

		dyapress_ecommerce.update_is_vat_applicable( form ); 
	    }
	} );

	$( '.dyapress-ecommerce-registration-purchase form .dyapress-ecommerce_country' ).change( function( event ) {
	    var form = $(this).parents( 'form' );
	    var country_id = parseInt( form.find('.dyapress-ecommerce_country').val() );
	    form.find( '.dyapress-ecommerce_vat_number' ).val( '' );
	    if ( dyapress_ecommerce.european_countries[country_id] ) {
		form.find( '.dyapress-ecommerce_field_vat' ).show();
	    } else {
		form.find( '.dyapress-ecommerce_field_vat' ).hide();
	    }
	    dyapress_ecommerce.update_is_vat_applicable( form ); 
	} );
	$( '.dyapress-ecommerce-registration-purchase form .dyapress-ecommerce_product_quantity' ).change( function( event ) {
	    dyapress_ecommerce.update_product_quantity( $(this) ); 
	} );
	$( '.dyapress-ecommerce-registration-purchase form .dyapress-ecommerce_vat' ).change( function( event ) {
	    var form = $(this).parents( 'form' );

	    dyapress_ecommerce.update_is_vat_applicable( form ); 
	} );

	$( '.dyapress-ecommerce-registration form' ).submit( function( event ) {
	    event.preventDefault();
	    var form = $( this );

	    form.fadeOut();
	    form.parent().find('.messages').html('<div class="info">' + dyapress_ecommerce.messages.processing + '</div>');

	    var param = new Object();
	    $( this ).find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each(function() {
		param[ $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' ) ] = $( this ).val();
	    });

	    $.post( dyapress_ecommerce.ajax_url, {
		action: 'dyapress_registration',
		param: param
	    }, function(response) {
		var result = JSON.parse(response);
		var html = '<div class="' + (result.success ? 'success' : 'failed') + '">' + result.message + '</div>';

		form.parent().find('.messages').html(html).format_prices();
	    } );
	} );

	$( '.dyapress-ecommerce-registration-purchase form' ).submit( function( event ) {
	    event.preventDefault();
	    var form = $( this );
	    var param = new Object();

	    form.find( 'button[type="submit"]' ).prop( "disabled", true ); // try to limit double click
	    form.find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each( function() {
		param[ $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' ) ] = $( this ).val();
	    } );
	    $.each( form.data(), function(k, v){
		if ( k.match( /^dyapress-ecommerce_/ ) && v )
		    param[ k.replace( 'dyapress-ecommerce_', '' ) ] = v;
	    } );

	    // Check if Stripe is used
	    if ( typeof dyapress_ecommerce.stripe != 'undefined' && typeof dyapress_ecommerce.card != 'undefined' ) {
		dyapress_ecommerce.card.update({value: {postalCode: form.find("input.dyapress-ecommerce_postcode").val()}});
		dyapress_ecommerce.stripe.createPaymentMethod('card', dyapress_ecommerce.card).then(function(result) {
		    if (result.error) {
			form.find(".stripe .card-errors").html(result.error.message).format_prices();
		    } else {
			param['stripe_payment_method_id'] = result.paymentMethod.id;
			dyapress_ecommerce.registration_purchase(form,param);
		    }
		} );
	    } else {
		dyapress_ecommerce.registration_purchase(form,param);
	    }
	} );


	$( '.dyapress-ecommerce-password form' ).submit( function( event ) {
	    event.preventDefault();
	    var form = $( this );

	    form.fadeOut();
	    form.parent().find('.messages').html('<div class="info">' + dyapress_ecommerce.messages.processing_update + '</div>');

	    var param = new Object();
	    $( this ).find( 'input[name^="dyapress-ecommerce_"], select[name^="dyapress-ecommerce_"], textarea[name^="dyapress-ecommerce_"]' ).each( function() {
		param[ $( this ).attr( 'name' ).replace( 'dyapress-ecommerce_', '' ) ] = $( this ).val();
	    } );

	    $.post( dyapress_ecommerce.ajax_url, {
		action: 'dyapress_reset_password',
		param: param
	    }, function(response) {
		var result = JSON.parse(response);
		var html = '<div class="' + (result.success ? 'success' : 'failed') + '">' + result.message + '</div>';

		form.fadeIn();
		form.parent().find('.messages').html(html).format_prices();
	    } );
	} );

	$( '#dyapress-ecommerce-account.vertical-tabs' ).tabs( {
	    beforeActivate: function( event, ui ){
		var name = ui.newPanel.attr( 'id' ).replace( 'tab-', '' );
		if ( name ) {
		    ui.newPanel.hide();
		    $.post( dyapress_ecommerce.ajax_url, {
			action: 'dyapress_get_account_section',
			name: name,
		    }, function(response) {
			var result = JSON.parse(response);
			ui.newPanel.html(result).format_prices();
			dyapress_ecommerce.account_init(); // FIXME: redo account initialization
			ui.newPanel.show();
		    } );
		}
	    },
	} );
	/* FIXME: keep accordion ?
	    $( '#dyapress-ecommerce-account.accordion' ).accordion( {
		beforeActivate: function( event, ui ){
		    if ( ui.newPanel.data( 'name' ) != null ) {
			ui.newPanel.height(0);
			$.post( dyapress_ecommerce.ajax_url, {
			    action: 'dyapress_get_account_section',
			    name: ui.newPanel.data( "name" ),
			}, function(response) {
			    console.log(response);

			    var result = JSON.parse(response);
			    ui.newPanel.html(result);
			    ui.newPanel.animate({ height: ui.newPanel.get(0).scrollHeight + "px" }, 300);
			} );
		    }
		},
	    } );
	    */
    } );

} )( jQuery );
