<?php
/*
 * DyaPress Premium Copyright(c) 2018-2020 Aytechnet https://www.aytechnet.fr/
 *
 * See https://dyapress.com/
 */

global $dyapress_ecommerce_args;
global $dyapress_ecommerce_current_user;

$products = dyapress_ecommerce()->decode_products( $dyapress_ecommerce_args );

if ( count($products) > 0 ) {
    $total_price_vat_excl = 0;
    $total_price_vat_incl = 0;

    foreach ( $products as $pid => $p ) {
	if ( !isset( $p ) ) { ?>
	    <p class="error"><?php printf( esc_html__( 'The product "%s" is not found', 'dyapress-ecommerce' ), $pid ); ?></p><?php
	} elseif ( isset( $p->hidden ) && $p->hidden ) {
	    $total_price_vat_incl += dyapress_premium()->get_total_price_vat_incl( $p->product, $p->quantity, $dyapress_ecommerce_args );
?>
	    <input type="hidden" class="dyapress-ecommerce_product_quantity"
		   data-product_price="<?php echo dyapress_premium()->get_product_price( $p->product, $p->quantity ); ?>"
		   data-product_vat="<?php echo dyapress_premium()->get_product_vat( $p->product ); ?>"
		   name="dyapress-ecommerce_product_quantity[<?php echo $pid; ?>]"
		   value="<?php echo $p->quantity; ?>" /><?php
	}
    } ?>
<table class="dyapress-ecommerce-prices products-table">
    <thead>
	<tr>
	    <th class="label"><?php _e( 'Description', 'dyapress-ecommerce' ); ?></th>
	    <th class="quantity"><?php _e( 'Quantity', 'dyapress-ecommerce' ); ?></th>
	    <th class="unit-price"><?php echo __( 'Unit price', 'dyapress-ecommerce' ) . dyapress_ecommerce()->format_vat_excl( '<br>' ); ?></th>
	    <th class="total-price"><?php echo __( 'Total', 'dyapress-ecommerce' ) . dyapress_ecommerce()->format_vat_excl( '<br>' ); ?></th>
	</tr>
    </thead>
    <tbody>
<?php
	foreach ( $products as $pid => $p ) {
	    if ( isset( $p ) && ( !isset( $p->hidden ) || !$p->hidden ) ) {
		$unit_price = dyapress_premium()->get_product_price_vat_excl( $p->product, $p->quantity );
		$product_price = dyapress_premium()->get_total_price_vat_excl( $p->product, $p->quantity );
		$total_price_vat_excl += dyapress_premium()->get_total_price_vat_excl( $p->product, $p->quantity, $dyapress_ecommerce_current_user );
		$total_price_vat_incl += dyapress_premium()->get_total_price_vat_incl( $p->product, $p->quantity, $dyapress_ecommerce_current_user );
?>
	<tr>
	    <td class="label"><?php echo apply_filters( 'dyapress_ecommerce_product_label', esc_html( $p->label ), $p ); ?></td>
	    <td class="field quantity field_product_quantity">
		<input type="number" class="dyapress-ecommerce_product_quantity"
		       data-product_price="<?php echo dyapress_premium()->get_product_price( $p->product, $p->quantity ); ?>"
		       data-product_vat="<?php echo dyapress_premium()->get_product_vat( $p->product ); ?>"
		       name="dyapress-ecommerce_product_quantity[<?php echo $pid; ?>]"
		       <?php echo $p->min > 0 ? 'min="'.$p->min.'"' : ''; ?>
		       <?php echo $p->max > 0 ? 'max="'.$p->max.'"' : ''; ?>
		       value="<?php echo $p->quantity; ?>" />
	    </td>
	    <td class="unit-price"><span class="unit-price-vat-excl__<?php echo $pid; ?>"><?php echo dyapress_ecommerce()->format_price($unit_price); ?></span></td>
	    <td class="total-price"><span class="total-price-vat-excl__<?php echo $pid; ?>"><?php echo dyapress_ecommerce()->format_price($product_price); ?></span></td>
	</tr>
<?php
	    }
	}
?>
    </tbody>
    <tfoot>
	<tr><td colspan="4">
	    <table class="summary">
		<tbody>
<?php if ( count( $products ) > 1 ) { ?>
		    <tr class="use-vat">
			<td class="label"><?php echo __( 'Total', 'dyapress-ecommerce' ) . dyapress_ecommerce()->format_vat_excl( '&nbsp;' ); ?></td>
			<td class="total-price total-price-vat-excl"><?php echo dyapress_ecommerce()->format_price( $total_price_vat_excl ); ?></td>
		    </tr>
<?php } ?>
		    <tr class="use-vat">
			<td class="label"><?php echo __( 'Total VAT', 'dyapress-ecommerce' ); ?></td>
			<td class="total-vat"><?php echo dyapress_ecommerce()->format_price( $total_price_vat_incl - $total_price_vat_excl ); ?></td>
		    </tr>
		    <tr class="total-to-pay">
			<td class="label"><?php echo __( 'Total', 'dyapress-ecommerce' ) . dyapress_ecommerce()->format_vat_incl( '&nbsp;' ); ?></td>
			<td class="total-price total-price-vat-incl"><?php echo dyapress_ecommerce()->format_price( $total_price_vat_incl ); ?></td>
		    </tr>
		</tbody>
	    </table>
	</td></tr>
    </tfoot>
</table>
<?php
}
